%define major 0

Name: openblas
Version: 0.3.19
Release: 1
License: BSD-3-Clause
Summary: Optimized BLAS Library
Url: https://openblas.net

BuildRequires: gcc10
BuildRequires: gcc10-cplusplus
BuildRequires: gcc10-gfortran
BuildRequires: libgomp-devel >= 6.3.0-12
BuildRequires: pase-build-tools
BuildRequires: binutils-gnu

Source0: https://github.com/xianyi/OpenBLAS/archive/v%{version}.tar.gz#/OpenBLAS-%{version}.tar.gz

Patch2: openblas-force-power10-support.patch
Patch3: openblas-aix-dynamic-cpu.patch
Patch4: openblas-power6-is-power7.patch
Patch5: openblas-aix-determine-num-threads-runtime.patch

%description
OpenBLAS is an optimized BLAS library based on GotoBLAS2 1.13 BSD version.

%package -n libopenblas%{major}
Summary:        OpenBLAS shared library

Requires: update-alternatives

Requires(post): update-alternatives
Requires(postun): update-alternatives

# Needed because automatic dependency generation doesn’t work with symlinks
Provides: lib:%{_libdir}/libblas-indirect.so.0(shr_64.o)(ppc64)
Provides: lib:%{_libdir}/libcblas-indirect.so.0(shr_64.o)(ppc64)
Provides: lib:%{_libdir}/liblapack-indirect.so.0(shr_64.o)(ppc64)
Provides: lib:%{_libdir}/liblapacke-indirect.so.0(shr_64.o)(ppc64)


%description -n libopenblas%{major}
OpenBLAS compression shared library.

%package devel
Summary:        Header files for OpenBLAS library
Requires:       libopenblas%{major} = %{version}-%{release}

%description devel
Header files for OpenBLAS library.


%prep

%setup -q -n OpenBLAS-%{version}

%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1

mkdir bin
ln -s %{_bindir}/gnu-as bin/as

%build

export PATH=$PWD/bin:$PATH

%make_build \
  DYNAMIC_ARCH=1 \
  BINARY=64 \
  OS=AIX \
  OSNAME=AIX \
  RANLIB=ranlib \
  CC=gcc-10 \
  FC=gfortran-10 \
  DYNAMIC_CORE="POWER6 POWER8 POWER9 POWER10" \
  TARGET=POWER6 \
  NUM_THREADS=64 \
  # end


%install 

perl -p -i -e 's|installbsd -c -m |install -pm|g' Makefile.install

%make_install \
  PREFIX=%{_prefix} \
  DESTDIR=%{buildroot} \
  OPENBLAS_INCLUDE_DIR=%{_includedir}/openblas # To avoid header file conflict with lapack 
      
LDFLAGS="-Wl,-bbigtoc -Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib -pthread -lgomp -lgfortran"
  
CC=g++-10 static-to-shared libopenblas.so.%{major} %{buildroot}%{_libdir}/libopenblas*%{version}.a $LDFLAGS

rm %{buildroot}%{_libdir}/libopenblas.so*

install -p -m 755 libopenblas.so.%{major} %{buildroot}%{_libdir}/libopenblas.so.%{major}

ln -s %{_libdir}/libopenblas.so.%{major} %{buildroot}%{_libdir}/libopenblas.so

rm %{buildroot}%{_libdir}/*.a

%post -n libopenblas%{major}
%{_bindir}/update-alternatives \
    --install %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libopenblas.so.%{major} 10 \
    --slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libopenblas.so.%{major} \
    --slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/libopenblas.so.%{major} \
    --slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/libopenblas.so.%{major} 

%postun -n libopenblas%{major}
%{_bindir}/update-alternatives \
--remove %{_libdir}/libblas-indirect.so.0 blas %{_libdir}/libopenblas.so.%{major} 10 \
--slave %{_libdir}/libcblas-indirect.so.0 cblas %{_libdir}/libopenblas.so.%{major} \
--slave %{_libdir}/liblapack-indirect.so.0 lapack %{_libdir}/libopenblas.so.%{major} \
--slave %{_libdir}/liblapacke-indirect.so.0 lapacke %{_libdir}/libopenblas.so.%{major} 


%files
%defattr(-, qsys, *none)

%license LICENSE
%doc README.md

%files -n lib%{name}%{major}
%defattr(-, qsys, *none)
%{_libdir}/libopenblas.so.%{major}
%ghost %{_libdir}/libblas-indirect.so.0
%ghost %{_libdir}/libcblas-indirect.so.0
%ghost %{_libdir}/liblapack-indirect.so.0
%ghost %{_libdir}/liblapacke-indirect.so.0

%license LICENSE
%doc README.md

%files devel
%defattr(-, qsys, *none)

%{_includedir}/openblas/cblas.h
%{_includedir}/openblas/f77blas.h
%{_includedir}/openblas/lapack.h
%{_includedir}/openblas/lapacke.h
%{_includedir}/openblas/lapacke_config.h
%{_includedir}/openblas/lapacke_mangling.h
%{_includedir}/openblas/lapacke_utils.h
%{_includedir}/openblas/openblas_config.h
%{_libdir}/cmake/openblas/*
%{_libdir}/libopenblas.so
%{_libdir}/pkgconfig/openblas.pc


%changelog
* Tue Dec 28 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.19
- Update to 0.3.19
- Improve source URL

* Fri Nov 19 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.18
- Add patch to determine the runtime threads count instead of defaulting to 2

* Thu Nov 18 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.18
- Explicitly set base target to POWER6 instead of using whatever the build
  system runs on
- Patch POWER6 flags to target POWER7 instead, since 7.3 doesn't run on P6
- Explicitly set thread count to support up to 64 threads instead of using
  whatever number of cores the build system has

* Tue Nov 16 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.18
- Set RANLIB to prevent build from using non-existent gcc-ranlib

* Fri Nov 12 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.18
- Update to 0.3.18 and enable POWER10 support
- Remove upstreamed patch
- Add patch to bypass invalid ld version check for P10 support
- Add patch for proper runtime CPU architecture detection on AIX

* Fri Nov 12 2021 Kevin Adler <kadler@us.ibm.com> - 0.3.12
- Build with GNU assembler and enable POWER9 support

* Thu Jan 28 2021 Naveen Ram <naveen.ram@ibm.com> - 0.3.12
- Set up indirects for cblas, lapack, lapacke

* Wed Nov 18 2020 - Naveen Ram <naveen.ram@ibm.com> - 0.3.12
- Fix typo and provide _SC_NPROCESSORS_ONLN support

* Mon Nov 9 2020 - Naveen Ram <naveen.ram@ibm.com> - 0.3.12
- Update to 0.3.12 and enable dynamic_arch

* Fri Oct 09 2020 - Naveen Ram <naveen.ram@ibm.com> - 0.3.10
- Create libblas-indirect for openblas

* Tue Aug 04 2020 - Naveen Ram <naveen.ram@ibm.com> - 0.3.10
- Create shared library from archive 

* Mon Jul 20 2020 - Naveen Ram <naveen.ram@ibm.com> - 0.3.10
- Initial package
